/* 
   postMessage() based communication channel between:
   - the host and a guestcode
   - two isolated guests
*/

(function(scope) {
    var channel = {};

    // Trusted domain names for communication
    // Should be configurable
    var whitelist = ["http://ads.com", "http://domain1"];

    /* origin and destination should be two window objects */
    channel.setup = function(origin) {
        setupHandler(origin);
        setupChannel(origin);
    };

    /*
      Setup a default message handler
    */
    function setupHandler(origin) {
        // no op. Default behavior to be overriden
        origin.handleEvent = function(event) {console.log(event.data)};
    };

    /* 
       Setup the postMessage channel for the destination window
       (listening to message) 
    */
    function setupChannel(destination) {

        destination.addEventListener("message", receiveMessage, false);
        
        function receiveMessage(event) {
            if (allowOrigin(event.origin)) {
                handleEvent(event);
            } else {
                alert('untrusted');
            }
        };

        function allowOrigin(origin) {
            var trusted = false;
            whitelist.forEach(function(each) {
                if(each === origin) { 
                    trusted = true;
                }
            });
            return trusted;
        };

        function handleEvent(event) {
            destination.handleEvent(event);
        };
    };

    scope.channel = channel;
})(this);
