/* 
   postMessage() based communication channel between:
   - the host and a guest
   - two isolated guests
*/

(function(scope) {
    var channel = {};

    // Trusted domain names for communication
    // TODO:
    // Should be configurable, using the service policy
    var whitelist = ["http://ads.com", "http://domain1"];

    // An array of all current communications
    var openCommunications = [];

    /* origin and destination should be two window objects */
    channel.setup = function(origin) {
        setupChannel(origin);
    };

    function openCommunication(destination, service) {
        //TODO: Shouldn't add a communication object if there's
        //already one for the same service, origin and destination
        var comm = scope.sandbox.communication(destination, service);
        openCommunications.push(comm);
    };

    function closeCommunication(communication) {
        //TODO
    };

    /* 
       Setup the postMessage channel for the destination window
       (listening to message) 
    */
    function setupChannel(destination) {

        destination.addEventListener("message", receiveMessage, false);
        
        function receiveMessage(event) {
            handleEvent(destination, event);
        };

        function allowOrigin(origin) {
            var trusted = false;
            whitelist.forEach(function(each) {
                if(each === origin) { 
                    trusted = true;
                }
            });
            return trusted;
        };

        function handleEvent(origin, event) {
            if(event.data.type === 'broadcast') {
                handleBroadcastEvent(origin, event);
            } else if(event.data.type === 'setup') {
                handleSetupEvent(origin, event);
            } else if(event.data.type === 'setupDone') {
                handleSetupDoneEvent(origin, event);
            } else {
                handleCommunication(origin, event);
            }
        };

        function serviceForEvent(origin, event) {
            var requestedService = event.data.service;
            var service;
            scope.sandbox.services.forEach(function(each) {
                if(each.domain === requestedService) {
                    service = each;
                }
            });

            return service;
        };

        function handleBroadcastEvent(origin, event) {
            origin.handleBroadcastEvent(event);
        };

        function handleSetupDoneEvent(origin, event) {
            var service = serviceForEvent(origin, event);
            if(service) {
                if (allowOrigin(event.origin)) {
                    openCommunication(origin, service);
                    handleCommunicationOpen(origin, service, event);
                } else {
                    console.log('untrusted. Will not create the service.');
                }
            }
        };

        function handleSetupEvent(origin, event) {
            var service = serviceForEvent(origin, event);
            if(service) {
                origin.handleSetupDoneEvent(event, service);
            }
        };

        //TODO: rename
        function handleCommunicationOpen(origin, service) {
            origin.handleCommunicationOpen(service);
        };

        function handleCommunication(origin, event) {
            var service = serviceForEvent(origin, event);
            if(service) {
                service.handleMessage(origin, event);
            } else {
                console.log('no such service');
            }
        };
    };

    scope.channel = channel;
})(this);
